/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "Attribute.h"
#include "Item.h"
#include "Cover.h"

/**
 * Outputs the attribute to a stream.
 * @param os The output stream.
 * @param attr The attribute.
 * @return The output stream.
 */
ostream& operator<<(ostream& os, Attribute& attr) {
	os << "c:" << attr.column << " " << attr.name << " -> ";
	bool first = true;
	for (map<string, Item*>::iterator iter = attr.items.begin(); iter != attr.items.end(); iter++) {
		if (!first) {
			os << std::endl;
			for (int i = 0; i < attr.name.length() + 8; i++) {
				os << " ";
			}
		}
		first = false;
		os << *(iter->second);
	}
	return os;
}

/**
 * Create a new attribute with name and column.
 * @param n The name.
 * @param c The column.
 */
Attribute::Attribute(string n, int c) {
	name = n;
	column = c;
}

/**
 * The destructor.
 */
Attribute::~Attribute() {
	for (map<string, Item*>::iterator iter = items.begin(); iter != items.end(); iter++) {
		delete iter->second;
	}
}

/**
 * Find an item stored in this attribute by its name.
 * @param value The unique item identifier.
 * @return The item with the identifier or NULL if this attribute has no such
 *         value (item).
 */
Item* Attribute::findItem(const string& value) {
	map<string, Item*>::iterator item = items.find(value);
	
	if (item == items.end()) {
		return NULL;
	} 
	
	return item->second;
}

/**
 * Inserts a value to this attribute or increases the support of an existing
 * value.
 * @param value The item value.
 * @param tid The transaction identifier of the transaction the value was on.
 */
void Attribute::addValue(const string& value, int tid) {
	map<string, Item*>::iterator item = items.find(value);
	
	if (item == items.end()) {
		items[value] = new Item(value, this, tid);
	} else {
		item->second->cover->addTransaction(tid);
	}
}
