/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_ATTRIBUTE
#define FIRM_ATTRIBUTE

#include <iostream>
using std::ostream;

#include <string>
using std::string;

#include <map>
using std::map;

class Item;

/**
 * An Attribute class for AVTree
 * @author Juho Muhonen
 * @modification 25.10.2005 commented all
 * @modification 23.3.2004 Class created
 */
class Attribute {
public:
	/**
	 * The name of the attribute.
	 */
	string name;

	/**
	 * The column of the attribute
	 */
	int column;
	
	/**
	 * Mapping from unique itemnames to Items
	 */
	map<string, Item*> items;
public:
	/**
	 * Create a new attribute with name and column.
	 */
	Attribute(string, int);
	
	/**
	 * The destructor.
	 */
	~Attribute();
	
	/**
	 * Find an item stored in this attribute by its name.
	 */
	Item* findItem(const string&);
	
	/**
	 * Adds an item to this attribute. If the item already exists.
	 * adds a transaction to that item.
	 */
	void Attribute::addValue(const string&, int);
	
	/**
	 * Outputs the attribute to a stream.
	 */
	friend ostream& operator<<(ostream&, Attribute&);
};

ostream& operator<<(ostream&, Attribute&);

#endif
