/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_CONTEXTMEDIANDURATIONCONSTRAINT
#define FIRM_CONTEXTMEDIANDURATIONCONSTRAINT

#include "SetConstraint.h"
#include "RuleConstraint.h"

class FITree;
class AssociationRule;
class ContextTree;

/**
 * A Class for computing the median of durations of a context itemset.
 * @author Juho Muhonen
 * @modified 7.11.2005 created
 */
class ContextMedianDurationConstraint:public SetConstraint, public RuleConstraint {
private:
	/**
	 * The duration of the last evaluated pattern.
	 */
	double last_duration;

	/**
	 * The minimum duration used in validating.
	 */
	double min_duration;
	
	/**
	 * The context tree.
	 */
	ContextTree* context_tree;	
public:	
	/**
	 * Create a median duration constraint with minimum duration of 0.
	 */
	ContextMedianDurationConstraint();
	
	/**
	 * Create a median duration constraint with minimum duration.
	 */
	ContextMedianDurationConstraint(double);
	
	/**
	 * Set the minimum duration of this constraint.
	 */
	void setMinDuration(double);
	
	/**
	 * Set the context dataset.
	 */
	void setContextTree(ContextTree*);

	/**
	 * Return the median duration of an itemset.
	 */
	double value(FITree*);

	/**
	 * Return the median duration of an association rule.
	 */
	double value(AssociationRule*);

	/**
	 * Validate an itemset.
	 */
	bool validate(FITree*);

	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum duration.
	 */
	double constraintLimit() const;

	/**
	 * Returns the name of this constraint.
	 */
	string name() const;

	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;

	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
