/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_FREQUENCYCONSTRAINT
#define FIRM_FREQUENCYCONSTRAINT

#include "SetConstraint.h"
#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for calculating the frequency of an itemset or an association rule.
 * @author Juho Muhonen
 * @modified 7.10.2005 commented all
 */
class FrequencyConstraint:public SetConstraint, public RuleConstraint {
private:
	/**
	 * The frequency of the last evaluated pattern.
	 */
	double last_frequency;
	
	/**
	 * The minimum frequency used in validating.
	 */
	double min_frequency;
		
public:	
	/**
	 * Create a frequency constraint with minimum frequency of 0
	 */
	FrequencyConstraint();
	
	/**
	 * Create a frequency constraint with a minimum frequency.
	 */
	FrequencyConstraint(double);
	
	/**
	 * Set the minimum frequency of this constraint.
	 */
	void setMinFrequency(double d);
	
	/**
	 * Return the frequency of an itemset.
	 */
	double value(FITree*);

	/**
	 * Return the frequency of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an itemset.
	 */
	bool validate(FITree*);

	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum frequency.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
