/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "Item.h"
#include "Attribute.h"
#include "Cover.h"

/**
 * Outputs the item to an output stream.
 * @param os The outputstream
 * @param item The item
 * @return The output stream
 */
ostream& operator<<(ostream& os, const Item& item) {
	if (item.attribute) {
		//os << (item.attribute)->name << ":'" << item.name << "'(" << (item.cover)->size << ")";
		os << (item.attribute)->name << ":'" << item.name << "'";
	} else {
		os << "NULL:'" << item.name << "'";	
	}
	return os;
}

/**
 * Compares two items by their cover size.
 * @param i The itemset to which this itemset is compared to.
 * @return The difference of the sizes of the covers of the itemset.
 * Negative if this itemset has smaller cover. If the covers are equally big
 * the address of the item is used as tie breaker to quarantee that the
 * re-evaluation of two unchanged items gives the same result.
 */
int Item::operator<(const Item& i) const {
	if (this->cover->size == i.cover->size) {
		return (this < &i);
	}
	return (this->cover->size < i.cover->size);
}


/**
 * Creates a fake item with name "X" and NULL attribute
 */
Item::Item() {
	attribute = NULL;
	name = "X";
}

/**
 * Creates a new item with name, attribute and transaction
 * @param n The name of the item.
 * @param a The attribute to which this item belongs to.
 * @param tid The transaction identifier that is added to the cover of this
 * item.
 */
Item::Item(string n, Attribute* a, int tid) {
	cover = new Cover();
	cover->addTransaction(tid);
	attribute = a;
	name = n;
}

/**
 * The destructor.
 */
Item::~Item() {
	delete cover;
}

/**
 * Compares two item pointers by the size of their covers.
 * @return See Item::operator<
 */
int item_pointer_compare(const Item* is1, const Item* is2) {
	return (*is1 < *is2);
}
