/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_ITEM
#define FIRM_ITEM

#include <iostream>
using std::ostream;

#include <string>
using std::string;

class Attribute;
class Cover;

/**
 * An item for itemset mining.
 * @author Juho Muhonen
 * @modification 6.10.2005 commented all
 */
class Item {
public:
	/**
	 * The attribute of the item.
	 */
	Attribute* attribute;
	
	/**
	 * The name of the item.
	 */
	string name;
	
	/**
	 * The cover of the item.
	 */
	Cover* cover;
		
public:
	/**
	 * Creates a fake item with name "X" and NULL attribute
	 */
	Item();
	
	/**
	 * Creates a new item with name, attribute and transaction
	 */
	Item(string, Attribute*, int);

	/**
	 * The destructor.
	 */
	~Item();

	/**
	 * Compares two items by their cover size.
	 */
	int operator<(const Item&) const;
	
	/**
	 * Outputs the item to an output stream.
	 */
	friend ostream& operator<<(ostream&, const Item&);
};

ostream& operator<<(ostream&, const Item&);

int item_pointer_compare(const Item*, const Item*);

#endif
