/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_NDI
#define FIRM_NDI

class FITree;
class AVTree;
class ContextTree;
class AssociationRule;
class Interval;

/**
 * This class is a home for functions for deriving the supports of itemsets
 * or confidences for association rules.
 * @author Juho Muhonen
 * @modification 10.10.2005 implemented context tree mining for ndi
 * @modification 6.10.2005 commented all
 */
class NDI {
public:
	/**
	 * Create a new NDI
	 */
	NDI();
	
	/**
	 * Destructor.
	 */
	~NDI();
	
	/**
	 * Generates a set of non-derivable frequent itemsets from a attribute-value dataset.
	 */
	FITree* generateNDI(AVTree*, int);

	/**
	 * Generates a set of non-derivable frequent itemsets from a context dataset.
	 */
	FITree* generateNDI(ContextTree*, int);

	/**
	 * Binds the support (finds the upper and lower bound of support) for an itemset
	 * given it's subsets.
	 */
	int boundSupport(FITree*);

	/**
	 * Binds the support of the condition of an association rule
	 * using the supports of its _available_ condition subrules given an association rule and the subsets of
	 * condition of that rule.
	 */
	int boundConditionSupport(AssociationRule*, vector<FITree*>* condition_subsets);

	/**
	 * Binds the support of the association rule using the _available_ condition subrules.
	 */
	Interval* boundConfidenceCondition(AssociationRule*);

	/**
	 * Binds the support of the association rule using the _available_ consequent subrules.
	 */
	Interval* boundConfidenceConsequent(AssociationRule*);

	/**
	 * Binds the confidence of an association rule given its available subrules.
	 */ 
	Interval* boundConfidence(AssociationRule*);
};

/**
 * A class for a interval.
 */
class Interval {
public:
	/**
	 * The lower bound of the interval.
	 */
	double lower_bound;
	
	/**
	 * The upper bound of the interval.
	 */
	double upper_bound;
	
public:
	/**
	 * Creates a new interval with lower bound of 0 and upper bound of 1.
	 */
	Interval();

	/**
	 * Creates a new interval with given loewr and upper bound.
	 */
	Interval(double, double);
	
	/**
	 * Destructor.
	 */
	~Interval();
};

#endif
