/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_PHPOUTPUTGENERATOR
#define FIRM_PHPOUTPUTGENERATOR

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <fstream>
using std::ofstream;

#include <iostream>
using std::ostream;

#include "OutputGenerator.h"

/**
 * A class for outputting association rules and itemsets in php format.
 * @author Juho Muhonen
 * @modified 31.10.2005 added support for precision
 * @modified 24.10.2005 created
 */
class PhpOutputGenerator:public OutputGenerator {
private:
	/**
	 * The main stream
	 */
	ostream* output_stream;
	
	
	/**
	 * Name of the main output file.
	 */
	string output_name;

	/**
	 * The set constraints.
	 */
	vector<SetConstraint*> set_constraints;
	
	/**
	 * The rule constraints.
	 */
	vector<RuleConstraint*> rule_constraints;
	
	/**
	 * Prints a double in color (html) using default values.
	 */
	void doubleInColor(double);
	
	/**
	 * Prints a double in color (html) using given upper and lower bound, zero limit, spacing and increasing
	 */
	void doubleInColor(double, double, double, double, int, bool);
	
	/**
	 * Outputs the header
	 */
	void outputHeader();
	
	/**
	 * Outputs the php sorting script
	 */
	void outputPhpSort();

	/**
	 * Outputs the top part for itemset output.
	 */
	void outputItemsetMeasures();
	
	/**
	 * Outputs the top part for association rule output.
	 */
	void outputRuleMeasures();
	
public:
	/**
	 * Creates a new html output generator.
	 */
	PhpOutputGenerator();
	
	/**
	 * Set the output name.
	 */
	void setOutputName(string);
	
	/**
	 * Add a set constraint to this output generator.
	 */
	void addSetConstraint(SetConstraint*);
	
	/**
	 * Add a rule constraint to this output generator.
	 */
	void addRuleConstraint(RuleConstraint*);

	/**
	 * Removes a constraint by it's name.
	 */
	void removeConstraint(string);

	/**
	 * Outputs an itemset.
	 */
	void outputSet(FITree*);

	/**
	 * Outputs a set of itemsets.
	 */
	void outputSets(FITree*);

	/**
	 * Outputs an association rule.
	 */
	void outputRule(AssociationRule*);

	/**
	 * Outputs a set of association rules.
	 */
	void outputRules(FITree*);

	/**
	 * Finalizes the generator.
	 */
	void finalize();
};
#endif
