/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_RULEGENERATOR
#define FIRM_RULEGENERATOR

#include <iostream>
using std::ostream;

#include <vector>
using std::vector;

class FITree;
class Item;

/**
 * Class for generating and outputting non-derivable association rules
 * @author Juho Muhonen
 * @modification 6.10.2005 commented all
 */
class RuleGenerator {
private:
	/**
	 * The root of the frequent itemset tree.
	 */
	FITree* root;
	
	/**
	 * Number of rules generated.
	 */
	int rules_generated;

	/**
	 * Number of rules output.
	 */
	int rules_output;

	/**
	 * Number of rules with zero difference of bounds.
	 */
	int rules_with_zero_dob;

	/**
	 * Number of rules with higher than minimum confidence.
	 */
	int rules_with_high_confidence;

	/**
	 * Minimum difference of bounds.
	 */
	double min_difference_of_bounds;

	/**
	 * Minimum confidence.
	 */
	double min_confidence;
	
public:
	/**
	 * Creates a new rule generator.
	 */
	RuleGenerator();

	/**
	 * The destructor.
	 */
	~RuleGenerator();

	/**
	 * Generates and counts non-derivable rules from a frequent itemset tree.
	 */	
	int generateRules(FITree*);

	/**
	 * Generates, counts and outputs rules from a frequent itemset tree to an output stream.
	 */	
	int generateRules(FITree*, ostream&);
	
	/**
	 * Return the number of rules generated.
	 */
	int rulesGenerated();

	/**
	 * Return the number of rules output.
	 */
	int rulesOutput();
	
	/**
	 * Return the number of confident rules.
	 */
	int rulesConfident();
	
	/**
	 * Return the number of rules with zero difference of bounds.
	 */
	int rulesWithZeroDob();
	
	/**
	 * Set the minimum difference of bounds.
	 */
	void setMinDifferenceOfBounds(double);

	/**
	 * Get the minimum difference of bounds.
	 */
	double getMinDifferenceOfBounds();

	/**
	 * Set the minimum confidence.
	 */
	void setMinConfidence(double);

	/**
	 * Get the minimum confidence
	 */
	double getMinConfidence();
};
#endif
