/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_SIZECONSTRAINT
#define FIRM_SIZECONSTRAINT

#include "SetConstraint.h"
#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for displaying the size of the itemset or association rule
 * @author Juho Muhonen
 * @modified 31.10.2005 created
 */
class SizeConstraint:public SetConstraint, public RuleConstraint {
private:
	/**
	 * The size of the last evaluated pattern.
	 */
	int last_size;
	
	/**
	 * The maximum size used in validating.
	 */
	int max_size;
		
public:	
	/**
	 * Create a size constraint with infinite maximum size
	 */
	SizeConstraint();
	
	/**
	 * Create a size constraint with a maximum size.
	 */
	SizeConstraint(int);
	
	/**
	 * Set the maximum size of this constraint.
	 */
	void setMaxSize(int);
	
	/**
	 * Return the size of an itemset.
	 */
	double value(FITree*);

	/**
	 * Return the size of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an itemset.
	 */
	bool validate(FITree*);

	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the maximum size.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns false.
	 */
	bool increasing() const;

	/**
	 * Returns 0.
	 */
	int precision() const;

	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
