/**
 * Copyright(c) Juho Muhonen 2004-2005
 * 
 * This file is part of 'firm'.
 * 
 * 'firm' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 'firm' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with 'firm'; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FIRM_SUPPORTCONSTRAINT
#define FIRM_SUPPORTCONSTRAINT

#include "SetConstraint.h"
#include "RuleConstraint.h"

class FITree;
class AssociationRule;

/**
 * A Class for computing the support of an itemset or an association rule.
 * @author Juho Muhonen
 * @modified 31.10.2005 precision()
 * @modified 25.10.2005 created
 */
class SupportConstraint:public SetConstraint, public RuleConstraint {
private:
	/**
	 * The support of the last evaluated pattern.
	 */
	int last_support;
	
	/**
	 * The minimum support used in validating.
	 */
	int min_support;
		
public:	
	/**
	 * Create a support constraint with minimum support of 0
	 */
	SupportConstraint();
	
	/**
	 * Create a support constraint with a minimum support.
	 */
	SupportConstraint(int);
	
	/**
	 * Set the minimum support of this constraint.
	 */
	void setMinSupport(int d);
	
	/**
	 * Return the support of an itemset.
	 */
	double value(FITree*);

	/**
	 * Return the support of an association rule.
	 */
	double value(AssociationRule*);
	
	/**
	 * Validate an itemset.
	 */
	bool validate(FITree*);

	/**
	 * Validate an association rule.
	 */
	bool validate(AssociationRule*);
	
	/**
	 * Returns the result of last evaluation.
	 */
	double lastResult();
	
	/**
	 * Returns the minimum support.
	 */
	double constraintLimit() const;
	
	/**
	 * Returns 0.
	 */
	int precision() const;

	/**
	 * Returns the name of this constraint.
	 */
	string name() const;
	
	/**
	 * Returns the abbreviation of this constraint.
	 */
	string abbreviation() const;
	
	/**
	 * Returns the description of this constraint.
	 */
	string description() const;
};
#endif
